// Datei: I2C.C
//
// I2C-Bus Schreib/Leseroutinen
// Mit dem I2C-Bus Modul im PIC16F873
// Der PIC arbeitet als Master
// Als EEPROM kommt ein 24C32 zum Einsatz
//
// Holger Klabunde
// 02.05.2002
// Compiler CC5X

#include "prozess.h"      /* Der verwendete Prozessor */
#include "protos.h"

//I2C-Bus Adressen der angeschlossenen Bausteine
#define EEPROMWRITE  0xA0
#define EEPROMREAD   EEPROMWRITE+1

unsigned char I2C_Read();
void I2C_Write(unsigned char);

void I2C_STOP()
{
//SSPCON2
 PEN=1;      //Sende Stopcondition
 while(PEN); //warten bis Stopcondition beendet ist
}

void I2C_START()
{
//SSPCON2
 SEN=1;      //Sende Startcondition
 while(SEN); //warten bis Startcondition beendet ist
}

void I2C_REPSTART()
{
//SSPCON2
 RSEN=1;      //Sende RepeatedStartcondition
 while(RSEN); //warten bis RepeatedStartcondition beendet ist
}

//##################################
//Liest ein Zeichen aus dem EEPROM
//##################################
void ReadChar(unsigned char ch)
{
 unsigned char by,i;
 uns16 adr;

 adr=(uns16)ch * (uns16)5; //5 Bytes pro Zeichen
 adr++; //Der Font im EEPROM ist um ein Byte verrutscht. Warum ?
 
 I2C_START(); //Startbedingung senden

 ACKDT=0;              //ACK=0 senden fr mehrere Bytes empfangen
 I2C_Write(EEPROMWRITE); //I2C-Adresse bertragen
 by=(unsigned char)(adr>>8);
 I2C_Write(by); //Zu lesende Adresse setzen Highbyte
 by=(unsigned char)(adr&0xFF);
 I2C_Write(by); //Zu lesende Adresse setzen Lowbyte

 I2C_REPSTART(); //RepeatedStartbedingung senden um den Bus zu behalten
 I2C_Write(EEPROMREAD); //I2C-Adresse mit LesenBit senden

 for(i=5; i>0; i--) //Hm Bytes verkehrt herum
  {
   if(i==1) ACKDT=1; //beim letzten Byte kein ACK senden !
   by=I2C_Read(); //Byte abholen

   charbuff[i-1]=by;
  }

 I2C_STOP(); //bertragung beendet
}

//***************************
//Liest ein Byte
//***************************
unsigned char I2C_Read(void)
{
//SSPCON2
 RCEN=1;      //Empfang starten
 while(RCEN); //warten auf Empfang

//SSPCON2
 ACKEN=1;      //ACKDT Bit senden
 while(ACKEN); //warten bis ACK beendet ist
  
 return SSPBUF;
}

//***************************
//Schreibt ein Byte bitweise
//***************************
void I2C_Write(unsigned char by)
{
 SSPIF=0;
 SSPBUF=by; //Datenbertragung starten
//SSPSTAT
// while(BF); //Warten auf Ende der bertragung

// while(ACKSTAT); //Warten auf ACK vom Slave, geht nicht ???             
 while(SSPIF==0); //Warten auf ACK vom Slave             
}
